package com.backendsys.service.B2c;

import com.backendsys.entity.B2c.B2cMemberDTO;
import com.backendsys.utils.response.Result;
import jakarta.servlet.http.HttpServletRequest;

import java.util.List;
import java.util.Map;

public interface B2cMemberService {
    Map<String, Object> queryMemberList(Integer pageNum, Integer pageSize, B2cMemberDTO b2cMemberDTO);
    List<Map<String, Object>> queryMemberWithLogined(Integer pageNum, Integer pageSize, B2cMemberDTO b2cMemberDTO);
    Map<String, Object> queryMemberDetail(Long good_id);
    Map<String, Object> insertMember(B2cMemberDTO b2cMemberDTO);
    Map<String, Object> updateMember(B2cMemberDTO b2cMemberDTO);
    Map<String, Object> deleteMember(B2cMemberDTO b2cMemberDTO);
    Result login(HttpServletRequest request, B2cMemberDTO b2cMemberDTO);
}
